#Auteur : Julien Bernon, Lycée Louis Feuillade, Lunel (34), Académie de Montpellier
# coding: utf-8
from matplotlib import pyplot as plt
from nanpy import ArduinoApi,SerialManager
import serial.tools.list_ports as port_list

#Détection/sélection du port sur lequel est branché l'Arduino
# sous Windows il pourra y avoir plusieurs ports série détectés de COM1, COM2... il faudra répondre COM2 si l'arduino est sur le COM2
# sous Linux(Ubuntu...)/Unix(MacOS) il faudra répondre /dev/tty... 
ports = list(port_list.comports())
for p in ports:
    print (p)
port_arduino=input("Quel port choisissez-vous ?")
connection = SerialManager(device=port_arduino)
a = ArduinoApi(connection=connection)

#Définition du pin sur lequel va se faire la mesure
#avec nanpy, les pins analogiques A0 -> A5
#sont numérotés à la suite ports numériques, donc de 14 -> 19
port_mesure= 14#entre 14(A0) et 19(A5)
a.pinMode(port_mesure,a.INPUT) #Le port défini juste au-dessus est activé en lecture


#lecture de la tension à vide U0
tension_a_vide=False
while not tension_a_vide:
    input("Mesure de la tension de la pile à vide, appuyez sur entrée lorsque vous êtes prêt(e)")
    valeur=a.analogRead(port_mesure)
    U0=round(5*valeur/1023,3)
    tension_a_vide=(input("tension à vide : "+str(U0)+" V. (Y pour continuer, sinon nouvelle mesure)")=="Y")


tensions=[] #Cette liste va stocker toutes les tensions mesurées
resistances=[] #Cette liste va stocker toutes les resistances que vous allez tester

# Début boucle de mesure, l'utilisateur n'en sort que lorsqu'il répond N
teste=True
while teste :
    resistance=(int(input("Valeur de la résistance ?"))) #L'utilisateur saisit la valeur de la résistance Rext
    resistances.append(resistance) #Cette valeur est ajoutée à la liste des résistances mesurées
    valeur=a.analogRead(port_mesure) #On demande à l'Arduino de lire la valeur de Uext correspondant
    tension=round(5*valeur/1023,3) #On convertit cette valeur en tension
    tensions.append(tension) #On ajoute le résultat de la mesure à la liste des tensions mesurées
    print("Valeur mesurée",tension,"V") #On affiche le résultat, pour information
    teste=not (input("Continuer ? (répondre N pour arrêter)")=="N") #Si on continue les mesure, il suffit d'appuyer sur Entrée. Pour arrêter, il faut répondre N


"""
En cas de problème, ci-dessous des valeurs de secours sont disponibles.
Si besoin commenter (de la même façon que ce commentaire) tout le code au-dessus en mettant 3 guillemets doubles à la ligne 9 puis 3 autre à la ligne 47 
Décommenter ensuite les lignes 54-55-56
"""
#---------------Valeurs de secours-------------------#
#U0=4.526
#tensions=[3.93,3.754,3.465,2.81,2.566,2.234,1.784,1.5,1.124,0.645]
#resistances=[400,300,200,100,80,60,40,30,20,10]
#---------------Fin Valeurs de secours---------------#

etude=[]#définition de la fonction proportionnelle à Uext
for i in range(len(resistances)):
    etude.append(resistances[i]*(U0-tensions[i]))

#Les 3 lignes précédentes sont équivalentes à : etude=[resistances[i]*(U0-tensions[i]) for i in range(len(tensions))] 

def regression_lineaire(x,y):
#calcule le coefficient a de la droite y=ax par la méthode des moindres carrés
    s_xy=0
    for i in range(len(x)):
        s_xy+=x[i]*y[i]
    s_x2=0
    for i in x:
        s_x2+=i**2
    return s_xy/s_x2


a=regression_lineaire(tensions,etude) # détermination de la droite de régression --> valeur de Rint
#Affichage des résultats
print("tensions = ",tensions)
print("resistances = ",resistances)
#tracé du graphique
plt.plot((min(tensions),max(tensions)),(a*min(tensions),a*max(tensions)),color="blue")# tracé de la droite de régression à l'aide des 2 valeurs extrêmes
plt.plot(tensions,etude,"r.")# tracé des points de mesure
x=min(tensions)+0.01*(max(tensions)-min(tensions)) #ces 3 lignes se contentent d'afficher
y=min(etude)+0.99*(max(etude)-min(etude))          #l'équation de la droite à l'endroit voulu (en haut à gauche)
plt.text(x,y,"droite de régression y="+str(round(a,2))+"x",verticalalignment='center', fontsize=14, color='r')

plt.show() #Affichage du graphique
